@@ -1,4 +1,4 @@
 from django.conf import settings
 
 MOM_FOLDER = getattr(settings, 'MOM_FOLDER', 'mom_data')
-MOM_FILE = getattr(settings, 'MOM_FILE', 'mom.yml')
\ No newline at end of file
+MOM_FILE = getattr(settings, 'MOM_FILE', 'mom.yaml')
